#!/usr/bin/perl -w
###############################################################################
##    Copyright (C) 2002 by Eric Gerbier
##    Bug reports to: gerbier@users.sourceforge.net
##    $Id: afick-tk.pl,v 1.29 2004/01/28 12:56:50 gerbier Exp $
##
##    This program is free software; you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation; either version 2 of the License, or
##    (at your option) any later version.
##
##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
################################################################################
# this program just provide a graphical interface to afick
# it just launch afick commands
################################################################################
# rem : can not work in tainted mode
################################################################################

use strict;
use Getopt::Long;    # arg analysis

use Tk;              # interface
use Tk::Balloon;
use Tk::LabFrame;       # frame with a label
use Tk::Label;
use Tk::ItemStyle;
use Tk::ProgressBar;    # the progress bar
use Tk::ROText;         # to have read-only text
use Tk::Text;
use Tk::Tree;

# global var
my $version = "0.8.0";
my $change_text;        # text widget for change display
my $warning_text;       # text widget for warnings
my $progress_text;      # text widget for progress file name
my $progress_bar;       # progress bar
my $percent_text;       # text percent
my $balloon;            # context help
my $ToggleProgress   = 1;    # checkbox var
my $progress_percent = 0;    # progress bar var

# command arguments
my $configfile = '';
my $help       = '';
my $verbose    = '';
my $Version    = '';
my $Report_full_newdel;
my $Warn_missing_file;
my $Warn_dead_symlinks;
my $Running;
my $Timing;

# labels for Optionmenu widgets (option configuration)
my @verbose_values     = qw(verbose_config verbose noverbose);
my @missing_values     = qw(missing_files_config missing_files nomissing_files);
my @dead_sym_values    = qw(dead_symlinks_config dead_symlinks nodead_symlinks);
my @full_newdel_values = qw(full_newdel_config full_newdel nofull_newdel);
my @running_files_values =
  qw(running_files_config running_files norunning_files);
my @timing_values = qw(timing_config timing notiming);

# default file
# be careful : same code in afick.pl
my $default_conf;
if ( $^O =~ m/linux/ ) {
	$default_conf = "/etc/afick.conf";
}
elsif ( $^O =~ m/^MSWin/ ) {
	$default_conf = "xp.conf";
}
else {
	$default_conf = "afick.conf";
}

#####################################################################
sub separator($) {
	my $w = shift (@_);
	$w->Label( -text => '     ' )->pack( -side => "left" );
}
#####################################################################
# used to clear output screen before each command
sub clear_text($) {
	my $widget = shift (@_);
	$widget->delete( '1.0', 'end' );
}

#####################################################################
# print warnings in warning section and console
sub warning ($) {
	my $text = shift (@_);
	$warning_text->insert( 'end', "$text\n" );
	$warning_text->update();
	warn "$text\n";
}
#####################################################################
# used to spawn afick commands
sub do_action($) {
	my $arg = shift (@_);

	$arg .= " -c \"$configfile\"" if ($configfile);

	# force values (overload values in config file)
	if ( $verbose eq $verbose_values[1] ) {
		$arg .= " --verbose";
	}
	elsif ( $verbose eq $verbose_values[2] ) {
		$arg .= " --noverbose";
	}

	if ( $Warn_missing_file eq $missing_values[1] ) {
		$arg .= " --missing_files";
	}
	elsif ( $Warn_missing_file eq $missing_values[2] ) {
		$arg .= " --nomissing_files";
	}

	if ( $Report_full_newdel eq $full_newdel_values[1] ) {
		$arg .= " --full_newdel";
	}
	elsif ( $Report_full_newdel eq $full_newdel_values[2] ) {
		$arg .= " --nofull_newdel";
	}

	if ( $Warn_dead_symlinks eq $dead_sym_values[1] ) {
		$arg .= " --dead_symlinks";
	}
	elsif ( $Warn_dead_symlinks eq $dead_sym_values[2] ) {
		$arg .= " --nodead_symlinks";
	}

	if ( $Warn_dead_symlinks eq $dead_sym_values[1] ) {
		$arg .= " --dead_symlinks";
	}
	elsif ( $Warn_dead_symlinks eq $dead_sym_values[2] ) {
		$arg .= " --nodead_symlinks";
	}

	if ( $Running eq $running_files_values[1] ) {
		$arg .= " --running_files";
	}
	elsif ( $Running eq $running_files_values[2] ) {
		$arg .= " --norunning_files";
	}

	if ( $Timing eq $timing_values[1] ) {
		$arg .= " --timing";
	}
	elsif ( $Timing eq $timing_values[2] ) {
		$arg .= " --notiming";
	}

	# progress Checkbutton
	$arg .= " --progress" if ($ToggleProgress);

	print "arg = $arg \n" if ( $verbose eq $verbose_values[1] );

	if ( !open( ACTION, "afick.pl $arg  2>&1 |" ) ) {
		warning("can not execute afick.pl : $!");
		return;
	}
	else {

		# save current cursor
		my $cursor = $change_text->cget('-cursor');

		# now a waiting cursor
		$change_text->configure( -cursor => 'clock' );
		clear_text($change_text);
		$change_text->update();
		clear_text($warning_text);
		$warning_text->update();
		clear_text($progress_text);
		$progress_text->update();

		my $refresh = 0;
		my $nb      = 0;
		my $total   = 1;    # to avoid a divide by zero
		$progress_percent = 0;
		while (<ACTION>) {
			if ( $_ =~ m/^progress total (\d+)/ ) {

				# get number of file in database
				$total = $1;
			}
			elsif ( $_ =~ m/^progress (.*)/ ) {

				# get current scanned file
				$nb++;
				$progress_percent = ( $nb * 100 ) / $total;
				$progress_bar->update();

				my $txt = sprintf( "%02d%% (%d)", $progress_percent, $nb );
				clear_text($percent_text);
				$percent_text->insert( 'end', $txt );
				$percent_text->update();

				my $fic = $1;
				clear_text($progress_text);
				$progress_text->insert( 'end', $fic );
				$progress_text->update();
			}
			elsif ( $_ =~ m/^WARNING: (.*)/ ) {
				warning($1);
			}
			else {
				$change_text->insert( 'end', $_ );

				# rafraichissement regulier
				$refresh++;
				if ( $refresh == 10 ) {
					$refresh = 0;
					$change_text->update();
				}
			}
		}    # while
		close(ACTION);

		# go to end of both screens
		$change_text->see('end');
		$warning_text->see('end');

		# restore cursor
		$change_text->configure( -cursor => $cursor );
		clear_text($progress_text);
	}
}

#####################################################################
# display help page
sub do_help($) {
	my $main = shift;

	my $top = $main->Toplevel( -title => "help" );
	$top->Button( -text => "quit", -command => [ $top => 'destroy' ] )->pack();
	my $text = $top->Scrolled(
		'ROText',
		-scrollbars => 'e',
		-height     => 25,
		-width      => 128,
		-wrap       => 'word'
	)->pack( -side => "left", -expand => 1, -fill => 'both' );

	my $baratin =
	  "this is a graphical interface to afick (another file integrity checker)
to monitor file system changes

actions buttons :
-----------------
init : to create the database
update : compare and update the database
compare : compare the files with the database
print : print database content
tree-view : display the change in a tree view
save  : save output screen to a local file
load  : display a saved outpout in outpout screen
help : this page
about : legal informations
exit : to quit this interface

the change section :
--------------------
display file changes

the warning section :
---------------------
display afick errors and warnings

the progress section :
----------------------
can be activated/desactivated by the 'display progress' checkbutton
it is useful fo follow afick progress (compare on update mode only)
it display the currently scanned file
then a progress bar, from 0 to 100%, with a line each 10%

configuration section :
-----------------------
a first line to work on configuration file :
- configuration file : to enter a configuration file name
- select... : to select a configuration file with a fileselector widget
- view/edit : to view or edit the selected configuration file

then a line of radio-buttons for option, corresponding to command-line options :
- verbose : produce more verbose output (for debugging purpose)
- missing_files : warn about files in config file but missing
- dead_links : warn about dead symlinks
- full_newdel : produce a full report on new/deleted directories
- running_files : warn about files modified during program run
- timing : print timing statistics

each offers 3 choices :
- keep config file choice (*_config)
- force parameter
- force opposite parameter (no*)
";

	$text->insert( 'end', $baratin );
	$text->see('1.0');
}
#####################################################################
# display about page
sub do_about($) {
	my $main = shift;

	my $top = $main->Toplevel( -title => "about" );
	$top->Button( -text => "quit", -command => [ $top => 'destroy' ] )->pack();
	my $text = $top->Scrolled(
		'ROText',
		-scrollbars => 'e',
		-height     => 25,
		-width      => 128,
		-wrap       => 'word'
	)->pack( -side => "left", -expand => 1, -fill => 'both' );

	my $baratin = "afick-tk version $version : a graphical interface to afick\n
url  : http://afick.sourceforge.net
Copyright (c) 2002 Eric Gerbier <gerbier\@users.sourceforge.net>
send remarks or bug reports to gerbier\@users.sourceforge.net

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

";

	$text->insert( 'end', $baratin );
	$text->see('1.0');
}
#####################################################################
# display about page
sub do_wizard($) {
	my $main = shift;

	my $top = $main->Toplevel( -title => "wizard" );
	$top->Button( -text => "quit", -command => [ $top => 'destroy' ] )->pack();
	my $text = $top->Scrolled(
		'ROText',
		-scrollbars => 'e',
		-height     => 25,
		-width      => 128,
		-wrap       => 'word'
	)->pack( -side => "left", -expand => 1, -fill => 'both' );

	my $baratin = "How to use afick ?

First : create a config file according to your needs :
- afick provide 2 config file sample : xp.conf and linux.conf
- read configuration documentation (afick.conf man page or config.html)
- customize your config file with afick-tk or your favorite editor

Second : initiate your database :
- apply 'init' button

you are now ready for afick use : compare, update, print ...
";

	$text->insert( 'end', $baratin );
	$text->see('1.0');
}
#####################################################################
## begin tree section
#####################################################################
sub do_tree_help($) {
	my $main = shift;

	my $top = $main->Toplevel( -title => "tree help" );
	$top->Button( -text => "quit", -command => [ $top => 'destroy' ] )->pack();
	my $text = $top->Scrolled(
		'ROText',
		-scrollbars => 'e',
		-height     => 25,
		-width      => 128,
		-wrap       => 'word'
	)->pack( -side => "left", -expand => 1, -fill => 'both' );

	my $baratin = "afick tree view allow to see a tree of changed files

colors :
- green for new files
- red for file changes
- blue for deleted files
- yellow for dangling files

actions :
- simple click : display details on selected file
- double click : clear color
";

	$text->insert( 'end', $baratin );
	$text->see('1.0');
}
#####################################################################
sub create_tree_buttons($) {
	my $mw = shift (@_);
	my $frame_text = $mw->LabFrame( -label => "statistiques" );
	$frame_text->pack( -expand => 0, -fill => 'x' );

	# new files label
	$frame_text->Label( -text => "new" )->pack( -side => 'left' );
	my $wnew = $frame_text->ROText(
		-height     => 1,
		-width      => 7,
		-foreground => 'green'
	);
	$wnew->pack( -side => 'left' );
	$balloon->attach( $wnew, -balloonmsg => "number of new files" );

	# deleted files label
	$frame_text->Label( -text => "deleted" )->pack( -side => 'left' );
	my $wdel = $frame_text->ROText(
		-height     => 1,
		-width      => 7,
		-foreground => 'blue'
	);
	$wdel->pack( -side => 'left' );
	$balloon->attach( $wdel, -balloonmsg => "number of deleted files" );

	# changed files label
	$frame_text->Label( -text => "changed" )->pack( -side => 'left' );
	my $wmod = $frame_text->ROText(
		-height     => 1,
		-width      => 7,
		-foreground => 'red'
	);
	$wmod->pack( -side => 'left' );
	$balloon->attach( $wmod, -balloonmsg => "number of changed files" );

	# dangling files label
	$frame_text->Label( -text => "dangling" )->pack( -side => 'left' );
	my $wdang = $frame_text->ROText(
		-height     => 1,
		-width      => 7,
		-foreground => 'yellow'
	);
	$wdang->pack( -side => 'left' );
	$balloon->attach( $wdang, -balloonmsg => "number of dangling files" );

	separator($frame_text);

	# help button
	my $bhelptree = $frame_text->Button(
		-text    => "help",
		-command => sub { do_tree_help($mw) }
	)->pack( -side => "left" );
	$balloon->attach( $bhelptree, -balloonmsg => "display help on tree view" );

	#quit button
	my $bquit =
	  $frame_text->Button( -text => "quit", -command => [ $mw => 'destroy' ] )
	  ->pack( -side => "left" );
	$balloon->attach( $bquit, -balloonmsg => "quit tree view" );

	return ( $wnew, $wdel, $wmod, $wdang );
}
########################################################################
sub add_tree($$$) {
	my ( $w, $fulldir, $style ) = @_;

	my $parent = '/';
	if ( $^O eq 'MSWin32' ) {
		if ( $fulldir =~ s/^([a-z]:)//i ) {
			$parent = $1;
		}
	}

	add_to_tree( $w, $parent, $parent ) unless $w->infoExists($parent);

	my $cur_parent = $parent;
	my @dirs       = ($cur_parent);
	foreach my $name ( split ( /[\/\\]/, $fulldir ) ) {
		next unless length $name;
		push @dirs, $name;
		my $dir = join ( '/', @dirs );
		add_to_tree( $w, $dir, $name, $cur_parent )
		  unless $w->infoExists($dir);
		$cur_parent = $dir;
	}

	# set color
	my $entry = $parent . $fulldir;
	if ( defined $style ) {
		$w->entryconfigure( $entry, -style => $style );
	}
}

########################################################################
sub add_to_tree {
	my ( $w, $dir, $name, $parent ) = @_;

	my $mode = 'close';

	my @args = ( -text => $name, -data => $name );
	if ($parent) {    # Add in alphabetical order.
		foreach my $sib ( $w->infoChildren($parent) ) {
			if ( $sib gt $dir ) {
				push @args, ( -before => $sib );
				last;
			}
		}
	}

	$w->add( $dir, @args );
	$w->setmode( $dir, $mode );
}

#####################################################################
sub tree_parse($$$$$$) {
	my ( $w_maintree, $wnew, $wdel, $wmod, $wdang, $h_change ) = @_;
	my $nbnew = 0;
	my $nbdel = 0;
	my $nbmod = 0;
	my $nbdan = 0;

	my $name;
	my $summary = 1;
	my $logs    = $change_text->get( '1.0', 'end' );

	my @logs = split ( /\n/, $logs );

	foreach (@logs) {
		chomp();

		if ($summary) {

			# summary part
			if (m/^new.*: (.*)/) {
				add_tree( $w_maintree, $1, 'new' );
				$h_change->{$1}{type} = 'new';
				$nbnew++;
			}
			elsif (m/^deleted.*: (.*)/) {
				add_tree( $w_maintree, $1, 'deleted' );
				$h_change->{$1}{type} = 'deleted';
				$nbdel++;
			}
			elsif (m/^changed.*: (.*)/) {
				add_tree( $w_maintree, $1, 'changed' );
				$h_change->{$1}{type} = 'changed';
				$nbmod++;
			}
			elsif (m/^Dangling.*: (.*)/) {
				add_tree( $w_maintree, $1, 'dangling' );
				$h_change->{$1}{type} = 'dangling';
				$nbdan++;
			}
			elsif (m/^# detailed changes/) {

				# summary part end at first blank line
				$summary = 0;
			}
		}
		else {

			# detailed part
			if (m/^\w.*: (.*)/) {
				$name = $1;
			}
			elsif (m/\t(\w+)\s+: (.*)\t(.*)/) {
				my $field = $1;
				my $old   = $2;
				my $new   = $3;
				$h_change->{$name}{$field} = "$old\t$new";
			}
			elsif (m/\t(.*)\s+: (.*)/) {
				my $field = $1;
				my $old   = $2;
				$h_change->{$name}{$field} = "$old";
			}
		}
	}

	$w_maintree->autosetmode();

	# write in text box
	$wnew->insert( 'end',  $nbnew );
	$wdel->insert( 'end',  $nbdel );
	$wmod->insert( 'end',  $nbmod );
	$wdang->insert( 'end', $nbdan );
}
########################################################################
# to display changes info in display panel
sub tree_display_detail($$$) {
	my $d        = shift;
	my $w_detail = shift;
	my $h_change = shift;

	#print "debug : tree_display_detail d=$d\n";
	$d =~ s!//!/!;    # remove first //

	$w_detail->delete('all');
	my $item = $w_detail->addchild("");
	$w_detail->itemCreate( $item, 0, -itemtype => 'text', -text => $d );

	if ( exists $h_change->{$d} ) {
		my $var = $h_change->{$d};
		$w_detail->itemCreate(
			$item, 1,
			-itemtype => 'text',
			-text     => $var->{type}
		);
		foreach my $elem ( keys %$var ) {
			next if ( $elem eq 'type' );
			$item = $w_detail->addchild("");

			$w_detail->itemCreate(
				$item, 0,
				-itemtype => 'text',
				-text     => $elem
			);

			my @tab = split ( /\t/, $var->{$elem} );
			my $col = 1;
			foreach my $t (@tab) {
				$w_detail->itemCreate(
					$item, $col,
					-itemtype => 'text',
					-text     => $t
				);
				$col++;
			}
		}
	}
}
########################################################################
sub tree_remove_color($$) {
	my $d          = shift;
	my $w_maintree = shift;
	$w_maintree->entryconfigure( $d, -style => 'normal' );
}
########################################################################
#####################################################################
sub create_tree_widget($$) {
	my $mw       = shift (@_);
	my $h_change = shift (@_);

	my $w_maintree = $mw->Scrolled(
		'Tree',
		-itemtype   => 'text',
		-separator  => '/',
		-selectmode => 'single',
		-scrollbars => 'osoe',
		-width      => 35,
		-height     => 35
	);
	$balloon->attach( $w_maintree, -balloonmsg => "tree of changes files" );

	my $w_details = $mw->Scrolled(
		'HList',
		-header  => 1,
		-columns => 4,
		-width   => 100
	);
	$balloon->attach( $w_details,
		-balloonmsg => "details about changes files" );

	my @header = ( "filename / field", "old value", "new_value" );
	my $nbcol = scalar(@header) - 1;
	for ( 0 .. $nbcol ) {
		$w_details->header( 'create', $_, -text => $header[$_] );
	}

	# single click : display info
	$w_maintree->configure( -browsecmd =>
		  sub { tree_display_detail( $_[0], $w_details, $h_change ) } );

	# double click : remove color
	$w_maintree->configure(
		-command => sub { tree_remove_color( $_[0], $w_maintree ) } );

	$w_maintree->packAdjust( -side => 'left', -fill => 'both', -delay => 1 );
	$w_details->pack( -side => 'right', -fill => 'both', -expand => 1 );

	# creation des style
	$w_maintree->ItemStyle(
		'text',
		-stylename  => 'normal',
		-foreground => 'black',
	);

	$w_maintree->ItemStyle(
		'text',
		-stylename  => 'changed',
		-foreground => 'red',
	);

	$w_maintree->ItemStyle(
		'text',
		-stylename  => 'new',
		-foreground => 'green',
	);

	$w_maintree->ItemStyle(
		'text',
		-stylename  => 'deleted',
		-foreground => 'blue',
	);

	$w_maintree->ItemStyle(
		'text',
		-stylename  => 'dangling',
		-foreground => 'yellow',
	);
	return ( $w_maintree, $w_details );
}
#####################################################################
sub do_tree($) {
	my $main = shift;
	my %h_change;

	my $mw = $main->Toplevel( -title => 'afick tree view' );

	my ( $wnew, $wdel, $wmod, $wdang ) = create_tree_buttons($mw);
	my ( $w_maintree, $w_details ) = create_tree_widget( $mw, \%h_change );

	tree_parse( $w_maintree, $wnew, $wdel, $wmod, $wdang, \%h_change );
}
#####################################################################
##  end of tree section
#####################################################################
# a global var for do_save_log and do_load_log
my $log_types =
  [ [ 'log files', '.log' ], [ 'text files', '.txt' ], [ 'All Files', '*', ], ];
#####################################################################
# save change screen to a log file
sub do_save_log($) {
	my $main = shift;

	my @logs = $change_text->get( '1.0', 'end' );
	my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday ) = gmtime(time);
	$year += 1900;
	$mon  += 1;
	my $date    = $year . "_" . "$mon" . "_" . $mday . "_" . $hour . "_" . $min;
	my $logname = "afick." . $date . ".log";
	my $filename = $main->getSaveFile(
		-initialfile => "$logname",
		-filetypes   => $log_types
	);

	if ($filename) {
		print "log in $filename\n";
		if ( !open( LOG, ">", $filename ) ) {
			warning("can not write on $filename");
		}
		else {
			print LOG @logs;
			close LOG;
		}
	}
}
#####################################################################
# load a log file into change part
sub do_load_log($) {
	my $main = shift;

	my @logs = $change_text->get( '1.0', 'end' );
	my $filename = $main->getOpenFile( -filetypes => $log_types );
	if ($filename) {
		clear_text($change_text);
		clear_text($warning_text);

		if ( !open( LOG, "<", $filename ) ) {
			warning("can not read $filename");
		}
		else {
			while ( my $line = <LOG> ) {
				$change_text->insert( 'end', $line );
			}
			$change_text->see('1.0');
			close LOG;
		}
	}
}
#####################################################################
# a global var for select_config and save_config
my $conf_types = [ [ 'config files', '.conf' ], [ 'All Files', '*', ], ];
#####################################################################
# to select a config file
sub select_config($$) {
	my $FenetreP = shift;
	my $entree   = shift;

	my $filename = $FenetreP->getOpenFile( -filetypes => $conf_types );
	if ( defined $filename and $filename ne '' ) {
		$entree->delete( 0, 'end' );
		$entree->insert( 0, $filename );
		$entree->xview('end');
	}
}
#####################################################################
# to save a config file
sub save_config($$$) {
	my $main     = shift;
	my $text     = shift;
	my $top      = shift;
	my $initfile = $configfile;
	$initfile =~ s?/?\\?g;
	my $filename = $main->getSaveFile(
		-initialfile => "$initfile",
		-filetypes   => $conf_types
	);

	my @conf = $text->get( '1.0', 'end' );
	if ( !open( CONF, ">", $configfile ) ) {
		warning("can not write in $configfile : $!");
	}
	else {
		print CONF @conf;
		close CONF;
	}
	$top->destroy();
}
#####################################################################
# open selected config file
sub open_config($) {
	my $main = shift;

	$configfile = $configfile || $default_conf;

	my $top = $main->Toplevel( -title => $configfile );
	my $text = $top->Scrolled(
		'Text',
		-scrollbars => 'e',
		-height     => 25,
		-width      => 128,
		-wrap       => 'word'
	)->pack( -side => "left", -expand => 1, -fill => 'both' );

	my $SaveButton = $top->Button(
		-text    => "save",
		-command => sub { save_config( $main, $text, $top ) }
	);
	$SaveButton->pack();
	$top->Button( -text => "quit", -command => [ $top => 'destroy' ] )
	  ->pack( -after => $SaveButton );

	if ( !open( CONF, "<", $configfile ) ) {
		warning("can not read $configfile : $!");
		return;
	}
	else {
		while (<CONF>) {
			$text->insert( 'end', $_ );
		}
		close(CONF);
		$text->see('1.0');
	}

}
#####################################################################
sub usage($) {

	my $version = shift (@_);
	print <<EOHELP
afick-tk ($version) : a graphical interface to afick
Usage: afick-tk [options]
Options:
--config_file|-c config
	name of config file to use
--full_newdel|-f
	report full  information on new and deleted directories
--help|-h
	this help
--missing_files|-m
	warn about files declared in config files which does not exists
--running_files|-r
	warn about files modified during program run
--dead_symlinks|-s
	warn about dead symlinks
--timing|-t
	Print timing statistics
--verbose|-v
	verbose mode (for debugging purpose)
--version|-V
	print program version

#################################################################
This script is intended to provide a means for
detecting changes made to files, via a regular
comparison of MD5 hashes to an established baseline. 
#################################################################
Copyright (c) 2002 Eric Gerbier <gerbier\@users.sourceforge.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
#################################################################
EOHELP
}
#####################################################################
##                       main
#####################################################################

# options
Getopt::Long::Configure('no_ignore_case');
unless (
	GetOptions(
		"config_file|c=s"  => \$configfile,
		"full_newdel|f!"   => \$Report_full_newdel,
		"help|h"           => \$help,
		"missing_files|m!" => \$Warn_missing_file,
		"running_files|r!" => \$Running,
		"dead_symlinks|s!" => \$Warn_dead_symlinks,
		"timing|t!"        => \$Timing,
		"verbose|v!"       => \$verbose,
		"version|V"        => \$Version
	)
  )
{
	usage($version);

	# can not use warning : no widget this time
	die ("incorrect option !");
}

if ($help) {
	usage($version);
	exit;
}

if ($Version) {
	print "afick-tk  version $version\n";
	exit;
}

# tk page
my $main = MainWindow->new( -title => "afick-tk $version" );

$balloon = $main->Balloon();

my $side = "top";

# we will have
# frame 1 : action
# frame 2 : outpout
# frame 3 : warning
# frame 4 : progress bar
# frame 5 : configuration
# frame 6 : options

# frame 1 : actions
####################
my $frame1 = $main->LabFrame( -label => "actions", -labelside => "acrosstop" );
$frame1->pack( -side => $side, -expand => 0, -fill => 'x' );

my $binit =
  $frame1->Button( -text => "init", -command => sub { do_action("-i") } );
$binit->pack( -side => "left" );
$balloon->attach( $binit, -balloonmsg => "create database" );

my $bupdate =
  $frame1->Button( -text => "update", -command => sub { do_action("-u") } );
$bupdate->pack( -side => "left" );
$balloon->attach( $bupdate, -balloonmsg => "update database" );

my $bcomp =
  $frame1->Button( -text => "compare", -command => sub { do_action("-k") } );
$bcomp->pack( -side => "left" );
$balloon->attach( $bcomp, -balloonmsg => "compare with database" );

my $bprint =
  $frame1->Button( -text => "print", -command => sub { do_action("-p") } );
$bprint->pack( -side => "left" );
$balloon->attach( $bprint, -balloonmsg => "print database" );

separator($frame1);

my $btree =
  $frame1->Button( -text => "tree-view", -command => sub { do_tree($main) } )
  ->pack( -side => "left" );
$balloon->attach( $btree, -balloonmsg => "display changes in a tree view" );

my $bsave =
  $frame1->Button( -text => "save", -command => sub { do_save_log($main) } )
  ->pack( -side => "left" );
$balloon->attach( $bsave, -balloonmsg => "save log to file" );

my $bload =
  $frame1->Button( -text => "load", -command => sub { do_load_log($main) } )
  ->pack( -side => "left" );
$balloon->attach( $bload, -balloonmsg => "load previous log from file" );

separator($frame1);

my $bhelp =
  $frame1->Button( -text => "help", -command => sub { do_help($main) } )
  ->pack( -side => "left" );
$balloon->attach( $bhelp, -balloonmsg => "display help text" );

my $babout =
  $frame1->Button( -text => "about", -command => sub { do_about($main) } )
  ->pack( -side => "left" );
$balloon->attach( $babout,
	-balloonmsg => "display about text (author, licence ...)" );

my $bwiz =
  $frame1->Button( -text => "wizard", -command => sub { do_wizard($main) } )
  ->pack( -side => "left" );
$balloon->attach( $bwiz, -balloonmsg => "small wizard display" );

separator($frame1);

my $bexit =
  $frame1->Button( -text => "exit", -command => sub { exit } )
  ->pack( -side => "left" );
$balloon->attach( $bexit, -balloonmsg => "exit from afick-tk" );

# frame 2 : text vue
####################
my $frame2 =
  $main->LabFrame( -label => "changes section", -labelside => "acrosstop" );
$frame2->pack( -side => $side, -expand => 1, -fill => 'both' );

$change_text = $frame2->Scrolled(
	'ROText',
	-scrollbars => 'e',
	-height     => 20,
	-width      => 128,
	-wrap       => 'word'
)->pack( -side => $side, -expand => 1, -fill => 'both' );
$balloon->attach( $change_text, -balloonmsg => "change window" );

# frame 3 : warning vue
####################
my $frame3 =
  $main->LabFrame( -label => "warnings section", -labelside => "acrosstop" );
$frame3->pack( -side => $side, -expand => 0, -fill => 'x' );

$warning_text = $frame3->Scrolled(
	'ROText',
	-scrollbars => 'e',
	-height     => 5,
	-width      => 128,
	-wrap       => 'word'
)->pack( -side => $side, -expand => 1, -fill => 'both' );
$balloon->attach( $warning_text, -balloonmsg => "warning window" );

# frame 4 : progress
####################
my $frame4 =
  $main->LabFrame( -label => "progress section", -labelside => "acrosstop" );
$frame4->pack( -side => $side, -expand => 0, -fill => 'x' );

my $btoggleprogress = $frame4->Checkbutton(
	-text     => "display progress",
	-variable => \$ToggleProgress
)->pack( -side => $side );
$balloon->attach( $btoggleprogress, -balloonmsg => "toggle progress bar" );

$progress_text = $frame4->ROText(
	-height => 1,
	-width  => 128,
	-wrap   => 'word'
)->pack( -side => $side );
$balloon->attach( $progress_text,
	-balloonmsg => "display current scanned file" );

$percent_text = $frame4->ROText(
	-height => 1,
	-width  => 16,
	-wrap   => 'word'
)->pack( -side => "right" );
$balloon->attach( $percent_text, -balloonmsg => "display percent progress" );

$progress_bar = $frame4->ProgressBar(
	-length      => 780,
	-colors      => [ 0, 'green' ],
	-troughcolor => 'grey55',
	-variable    => \$progress_percent
)->pack( -side => $side );
$balloon->attach( $progress_bar, -balloonmsg => "progress bar" );

# frame 5  : configuration file
####################
my $frame5 = $main->LabFrame(
	-label     => "configuration section",
	-labelside => "acrosstop"
);
$frame5->pack( -side => $side, -expand => 0, -fill => 'x' );

$frame5->Label( -text => "configuration file" )->pack( -side => "left" );
my $entry =
  $frame5->Entry( -textvariable => \$configfile, -width => 30 )
  ->pack( -side => "left", -expand => 1 );
$balloon->attach( $entry, -balloonmsg => "enter a configuration file name" );

my $bselect = $frame5->Button(
	-text    => 'Select...',
	-command => sub { select_config( $main, $entry ) }
)->pack( -side => "left" );
$balloon->attach( $bselect,
	-balloonmsg => "select a configuration file from a file browse" );

my $bedit = $frame5->Button(
	-text    => 'View/Edit',
	-command => sub { open_config($main) }
)->pack( -side => "left" );
$balloon->attach( $bedit, -balloonmsg => "edit the configuration file" );

# frame 6 : options
###################
my $frame6 = $main->LabFrame( -label => "options", -labelside => "acrosstop" );
$frame6->pack( -side => $side, -expand => 0, -fill => 'x' );

my $bverbose = $frame6->Optionmenu(
	-options  => [@verbose_values],
	-variable => \$verbose
)->pack( -side => "left" );
$balloon->attach( $bverbose, -balloonmsg => "configure verbose parameter" );

my $bmissing = $frame6->Optionmenu(
	-options  => [@missing_values],
	-variable => \$Warn_missing_file
)->pack( -side => "left" );
$balloon->attach( $bmissing,
	-balloonmsg => "configure missing_files parameter" );

my $bsymlink = $frame6->Optionmenu(
	-options  => [@dead_sym_values],
	-variable => \$Warn_dead_symlinks
)->pack( -side => "left" );
$balloon->attach( $bsymlink,
	-balloonmsg => "configure dead_symlinks parameter" );

my $bfull = $frame6->Optionmenu(
	-options  => [@full_newdel_values],
	-variable => \$Report_full_newdel
)->pack( -side => "left" );
$balloon->attach( $bfull, -balloonmsg => "configure full_newdel parameter" );

my $brunning = $frame6->Optionmenu(
	-options  => [@running_files_values],
	-variable => \$Running
)->pack( -side => "left" );
$balloon->attach( $brunning,
	-balloonmsg => "configure running_files parameter" );

my $btiming = $frame6->Optionmenu(
	-options  => [@timing_values],
	-variable => \$Timing
)->pack( -side => "left" );
$balloon->attach( $btiming, -balloonmsg => "configure timing parameter" );

MainLoop;
